export const DIRECTIVE_MAP = {
	'=': 'equals',
	'!=': 'not-equals',
	'<': 'greater-than',
	'>': 'less-than',
	'<=': 'greater-than-equal',
	'>=': 'less-than-equal',
};

export const DATA_SOURCE_QUERY_OPERATORS = Object.keys(DIRECTIVE_MAP) as (keyof typeof DIRECTIVE_MAP)[];

export type DataSourceQueryOperator = typeof DATA_SOURCE_QUERY_OPERATORS[number];

export interface DataSourceQuery {
	lhs: string;
	operator: DataSourceQueryOperator;
	rhs: string;
}

interface ConvertToDataSourceQueryCollectionOptions {
}

const parseDirectives = (valueRaw: string) => {
	let operator = '=';

	let hasDirective: boolean;
	let valueTest = valueRaw.toLowerCase();
	let valuePredicate = valueRaw;
	const fragments = valueTest.split('.');
	const directiveShorthands = Object.values(DIRECTIVE_MAP).map((s) => {
		// TODO how to parse?
	})
	do {
		hasDirective = false;

		if (valueTest.startsWith('.neq.')) {
			operator = '!=';

			valueTest = valueTest.slice('.neq.'.length);
			valuePredicate = valuePredicate.slice('.neq.'.length);
			hasDirective = true;
		} else if (valueTest.startsWith('.gt.eq.')) {
			operator = '>=';

			valueTest = valueTest.slice('.gt.eq.'.length);
			valuePredicate = valuePredicate.slice('.gt.eq.'.length);
			hasDirective = true;
		} else if (valueTest.startsWith('.gte.')) {
			operator = '>=';

			valueTest = valueTest.slice('.gte.'.length);
			valuePredicate = valuePredicate.slice('.gte.'.length);
			hasDirective = true;
		} else if (valueTest.startsWith('.lt.eq.')) {
			operator = '<=';

			valueTest = valueTest.slice('.lt.eq.'.length);
			valuePredicate = valuePredicate.slice('.lt.eq.'.length);
			hasDirective = true;
		} else if (valueTest.startsWith('.lte.')) {
			operator = '<=';

			valueTest = valueTest.slice('.lte.'.length);
			valuePredicate = valuePredicate.slice('.lte.'.length);
			hasDirective = true;
		} else if (valueTest.startsWith('.gt.')) {
			operator = '>';

			valueTest = valueTest.slice('.gt.'.length);
			valuePredicate = valuePredicate.slice('.gt.'.length);
			hasDirective = true;
		} else if (valueTest.startsWith('.lt.')) {
			operator = '<';

			valueTest = valueTest.slice('.lt.'.length);
			valuePredicate = valuePredicate.slice('.lt.'.length);
			hasDirective = true;
		} else if (valueTest.startsWith('.eq.')) {
			operator = '=';

			valueTest = valueTest.slice('.eq.'.length);
			valuePredicate = valuePredicate.slice('.eq.'.length);
			hasDirective = true;
		}
	} while (hasDirective);

	return {
		operator,
		rhs: valuePredicate,
	};
};

export const convertToDataSourceQueryCollection = (
	q: URLSearchParams,
	options = {} as ConvertToDataSourceQueryCollectionOptions,
): DataSourceQuery[] => {
	return Array.from(q.entries())
		.reduce(
			(queries, [key, value]) => {
				const s = parseDirectives(value);

				return [
					...queries,
					{
						lhs: key,
						operator: s.operator,
						rhs: JSON.stringify(s.rhs),
					},
				];
			},
			[] as DataSourceQuery[]
		);
	// all the queries are to be treated as "AND", as suggested by the & character to separate the query param entries
};
